@@ -130,6 +130,9 @@
 ##'   recorded as day zero in the \code{designfile}.
 ##' @param outSymbol a character string. A common prefix exists in the names of
 ##'   output files.
+##' @param parallelize logical. If \code{TRUE}, computation will be done in paralleL
+##' Doesn't work in windows machine 
+##' @param nCores a integer. Bigger or equal to one, number of cores to use 
 ##' @return
 ##' \code{meta3d} will write analysis results to \code{outdir} instead of
 ##'   returning them as objects. Output files with "meta3dSubjectID" in
@@ -171,7 +174,8 @@ meta3d <- function(datafile, designfile, outdir="metaout", filestyle,
                     design_groupColm=NULL, design_libIDrename=NULL,
                     adjustPhase="predictedPer", combinePvalue="fisher",
                     weightedMethod=TRUE, outIntegration="both", ARSmle="auto",
-                    ARSdefaultPer=24, dayZeroBased=FALSE, outSymbol="")
+                    ARSdefaultPer=24, dayZeroBased=FALSE, outSymbol="", 
+                    parallelize=FALSE, nCores=1)
 {
 
     ####start time
@@ -495,8 +499,32 @@ meta3d <- function(datafile, designfile, outdir="metaout", filestyle,
 
     ###run 'meta2d()' function subject by subject
     SUBJECT_LSL <- list();
-    for (i in 1:length(UNI_SUBJECTID))
-    {
+    if(parallelize){
+      SUBJECT_LSL <- parallel::mcmapply(
+        function(i){
+          subject_designD <- SUBJECT_DGL[[i]];
+          subject_libID <- subject_designD$libID;
+          subject_timeT <- subject_designD$timeT;
+          ###sort time points in order
+          names(subject_timeT) <- 1:length(subject_timeT);
+          order_timeT <- names(sort(subject_timeT));
+          order_timeT <- as.numeric(order_timeT);
+          ###arrange libraryID of each subject according sorted time points
+          expdataD <- EXPD[,subject_libID[order_timeT]];
+          dimnames(expdataD)[[1]] <- EXP_IDS;
+          dimnames(expdataD)[[2]] <- subject_libID[order_timeT];
+          meta2d_outD <- runmeta2dF(indata=expdataD, intime=subject_timeT[order_timeT],
+                                    datatype=patternD[i,], rundir=OUTDIR, runMethod=cycMethodOne,
+                                    minper=minper, maxper=maxper, ARSmle=ARSmle, ARSdefaultPer=ARSdefaultPer, para = F, nCores = 1);
+          return(meta2d_outD) 
+          
+        }
+        ,1:length(UNI_SUBJECTID), SIMPLIFY = FALSE ,mc.cores = nCores
+      )
+      a <- 1
+    } else {
+      for (i in 1:length(UNI_SUBJECTID))
+      {
         subject_designD <- SUBJECT_DGL[[i]];
         subject_libID <- subject_designD$libID;
         subject_timeT <- subject_designD$timeT;
@@ -511,11 +539,12 @@ meta3d <- function(datafile, designfile, outdir="metaout", filestyle,
         cat(paste("The 'meta3d' is processing ", UNI_SUBJECTID[i],
                   ", the ", i, " in total ", length(UNI_SUBJECTID),
                   " subjects.\n", sep="") );
-
+        
         meta2d_outD <- runmeta2dF(indata=expdataD, intime=subject_timeT[order_timeT],
-                            datatype=patternD[i,], rundir=OUTDIR, runMethod=cycMethodOne,
-                            minper=minper, maxper=maxper, ARSmle=ARSmle, ARSdefaultPer=ARSdefaultPer);
+                                  datatype=patternD[i,], rundir=OUTDIR, runMethod=cycMethodOne,
+                                  minper=minper, maxper=maxper, ARSmle=ARSmle, ARSdefaultPer=ARSdefaultPer, para = F, nCores = 1);
         SUBJECT_LSL[[i]] <- meta2d_outD;
+      }
     }
 
     ###check whether there are analysis results
