@@ -1,6 +1,4 @@
 ###### This file contains functions used in 'meta3dMainF.R'
-###### Author: Gang Wu (wggucas@gmail.com)
-###### Lab: John Hogenesch's lab (http://hogeneschlab.org/)
 ######======================================================================================================================================
 ##extract time information from given columns
 extractTimeColF <- function(design_timeColm, notekeys=c("design_hrColm", "hour"), designD, IDS) 
@@ -184,7 +182,7 @@ addMissingValueF <- function(dataD, rawtime, duplicate)
 ##the "intime" is already sorted in order; "datatype" is a vector containing five elements-
 ##nonInteger, uneven, miss, dup, uniNum, see the return values of "checkDataTypeF"
 runmeta2dF <- function(indata, intime, datatype, rundir, runMethod="JTK", 
-                       minper=20, maxper=28, ARSmle="auto", ARSdefaultPer=24)
+                       minper=20, maxper=28, ARSmle="auto", ARSdefaultPer=24, para = F, nCores = 1)
 {
 	meta2data <- indata
 	meta2time <- intime
@@ -202,17 +200,17 @@ runmeta2dF <- function(indata, intime, datatype, rundir, runMethod="JTK",
 		meta2data <- addNA$out
 		meta2time <- addNA$tim
 	}
-
 	##write 'meta2data' to a temporary file 
-	tempname <- paste("runmeta2dtemp", as.character(runif(1,0,1)), sep="")
-	tempfname <- paste(rundir, .Platform$file.sep, tempname, sep="")
+	## use process id to avoid duplicate file name when parallel
+	tempfname <- tempfile(pattern=paste("file", Sys.getpid(), sep=""), tmpdir=paste(rundir, .Platform$file.sep, sep="") ,fileext = '.txt')
+	
 	write.table(meta2data, file=tempfname, quote=FALSE, sep="\t", row.names=FALSE)	
 	##run selected method with 'meta2d()' function	
 	outL <- meta2d(infile=tempfname, outdir=rundir, filestyle="txt", timepoints=meta2time,
                 minper=minper, maxper=maxper, cycMethod=runMethod, adjustPhase="predictedPer", 
 				combinePvalue="fisher", analysisStrategy="auto", weightedPerPha=FALSE,
                 outputFile=FALSE, ARSmle=ARSmle, outIntegration="onlyIntegration", 
-                ARSdefaultPer=ARSdefaultPer, outRawData=TRUE, releaseNote=FALSE)
+                ARSdefaultPer=ARSdefaultPer, outRawData=TRUE, releaseNote=FALSE, parallelize = para, nCores = nCores)
     outD <- outL$meta
 	##delete the temporary file and return results
 	file.remove(tempfname)
